'use strict';

const util = require('util');
const DBusProxy = require("./DBusProxy.js");
const Log = require("../../../utils/log");
const TAG = "DMPProxy";

var DMPProxy = function() {
    this.proxy = new DBusProxy('iot.dmp.dimu', '/iot/dmp/dimu',
            'iot.dmp.dimu');
}

DMPProxy.prototype.isOnline = function() {
    return DBusProxy.isServiceOnline("iot.dmp.dimu");
};

DMPProxy.prototype.getDeviceList = function(status) {
    var list;
    if (status == null) {
        list = this.proxy.callService('getDeviceList', "");
    } else if (typeof status === 'string' && (status === 'online' || status === 'offline')) {
        var args = util.format("deviceState=%s", status);
        list = this.proxy.callService('getDeviceList', args);
    } else {
        throw new Error("the parameters must be 'online', 'offline', or undefined.");
    }

    try {
        console.log(util.format("DMPProxy.getDeviceList('%s') = %s", status, list));
        var arr = JSON.parse(list);
        return arr;
    } catch (e) {
        console.log("[DMPProxy.getDeviceList] parse device list failed : " + e.message);
    }
    return undefined;
}

DMPProxy.prototype.getDeviceListAsync = function(status, callback) {
    var callback_handler;
    var handleResult = function(err, info) {
        if (err) {
            callback_handler(err, undefined);
        } else {
            var result;
            try {
                result = JSON.parse(info);
            } catch (e) {
                console.log("[DMPProxy.getDeviceList] parse device list failed : " + e.message);
                result = undefined;
                err = new Error("return value can not be parsed as JSON.");
            }
            callback_handler(err, result);
        }
    };

    if (typeof status === 'function') {
        callback_handler = status;
        this.proxy.callServiceAsync('getDeviceList', handleResult);
    } else if (typeof status === 'string' && (status === 'online' || status === 'offline')) {
        callback_handler = callback;
        var args = util.format("deviceState=%s", status);
        this.proxy.callServiceAsync('getDeviceList', args, handleResult);
    } else if (status == null){
        callback_handler = callback;
        this.proxy.callServiceAsync('getDeviceList', '', handleResult);
    }else {
        throw new Error("the parameters must be 'online', 'offline', or undefined.");
    }
}
/**
 * 读取设备的profile
 * @param  {[String]} cloudid [设备的id]
 * @return {[Object]}         [返回的对象，格式如下：
 *                         {
                            "code": 0, // 状态码
                            "message": "success",
                            "params": {
                                "profile": profileOfDevice // profile数据
                            }
                        }
 * ]
 */
DMPProxy.prototype.getDeviceProfile = function(cloudid) {
     if (cloudid == null) {
         throw new Error('cloudid is not defined.');
     }

     var result = this.proxy.callService('getDeviceProfile', cloudid);
     try {
         var obj = JSON.parse(result);
         return obj;
    } catch (e) {
        console.log("[DMPProxy.getDeviceProfile] parse result failed: " + e.message);
    }
    return undefined;
};

DMPProxy.prototype.getDeviceProfileAsync = function(cloudid, callback) {
    Log.i(TAG, "getDeviceProfileAsync begin id = " + cloudid);
    if (cloudid == null) {
        throw new Error('cloudid is not defined.');
    }
    this.proxy.callServiceAsync('getDeviceProfile', cloudid, function(err, value) {
        Log.i(TAG, "getDeviceProfileAsync res id = " + cloudid + " err " + err+ " value = " + value);
        if (err) {
            callback(err, undefined);
        } else {
            var result;
            try {
                result = JSON.parse(value);
            } catch (e) {
                console.log("[DMPProxy.getDeviceProfileAsync] parse result failed: " + e.message);
                result = undefined;
                err = new Error("return value can not be parsed as JSON.");
            }
            callback(err, result);
        }
    });
};

module.exports = new DMPProxy();

