'use strict';
const Log = require("../../utils/log");
const constant = require("../../utils/constant");
const TAG = "[Trigger] ";
const RUN_TAG = "[FC_Running_Begin]";
var nodeSchedule = require("node-schedule");
const groupAPI = require("../../runner/developer/api-group");
const fs = require("fs");
const path = require('path');
var child_process = require("child_process");
const {PROJECT_PATH} = require('../../const');
const dirname = path.join(PROJECT_PATH, 'components/fc');


var Trigger = function (job, functionService, callback) {
  Log.i(TAG, "new Trigger begin...");
  this.functionService = functionService;
  this.job = JSON.parse(JSON.stringify(job));
  this.jobId = job.jobId;
  this.ackCode = job.ackCode;
  if (this.job.path && this.job.path.startsWith("./")) {
    this.job.path = path.join(dirname, "../..", this.job.path.split("./")[1]);
  }
  this.config = this.getConfigObject();
  try {
    this.functionId = this.config[constant.CONFIG_CONFIG][constant.CONFIG_FUNCTIONID];
    this.runMode = this.config[constant.CONFIG_CONFIG][constant.CONFIG_RUNMODE];
  } catch (err) {
    Log.e(TAG, "err", err);
  }
  this.triggers = this.getTrigger(this.config);
  this.groupId = this.getGroupId();
  // if(!this.checkError()) {
  //     this.startCrontab();
  // }
  var self = this;
  groupAPI.getConfigByGroupId(this.groupId, function (err, config) {
    Log.i(TAG, "getConfigByGroupId err = " + err + "config = " + JSON.stringify(config));
    if (err) {
      config = null;
    }
    self.groupConfig = config;
    self.updateTriggerIds(self.groupConfig);
    callback(null, self);
  });
};

Trigger.TRIGGER_TYPE = {
  event: 1,
  propertyChange: 2
};

Trigger.prototype.execute = function () {
  delete(this.job.ackId);
  var params = {
    functionId: this.functionId,
    invocationType: 'Async',
    invokerContext: this.job.context,
    payload: this.job.event,
  };
  Log.i(TAG, `Invoke function ${this.functionId}.`);
  this.functionService.invokeFunction(params, function (err, invocationId) {
    if (err) {
      Log.i(TAG, 'Failed to invoke function,' + err);
    } else {
      Log.i(TAG, `Invocation id is ${invocationId}.`);
    }
  });
};

Trigger.prototype.checkError = function () {
  Log.i(TAG, "checkError called.");
  var jobPath = this.job.path;
  Log.i(TAG, "Sctipt runScript begin jobPath = " + jobPath);
  var code;
  var config = jobPath + "/config.json";
  try {
    if (!fs.existsSync(config)) {
      return new Error(`Can not find config: ${config}.`);
    }
    if (this.job.type === 'zip') {
      var handler = (this.config)[constant.CONFIG_CONFIG][constant.CONFIG_HANDLER];
      var runtime = (this.config)[constant.CONFIG_CONFIG][constant.CONFIG_FCRUNTIME];
      var suffix;
      if (runtime === 'nodejs8') {
        suffix = '.js';
      } else if (runtime === 'python3') {
        suffix = '.py';
      } else {
        return new Error(`Unsupported runtime ${runtime}.`);
      }
      code = jobPath + "/" + handler.split(".")[0] + suffix;
    } else {
      code = jobPath + "/" + constant.jobIndex;
    }
    if (!fs.existsSync(code)) {
      return new Error(`Can not find module: ${code}.`);
    }
  } catch (err) {
    Log.e("checkError err = " + err);
    return err;
  }
  return null;
};

/**
 * 脚本是否包含trigger
 * @return {*|boolean}
 */
Trigger.prototype.hasTrigger = function () {
  Log.i(TAG, "triggers = " + JSON.stringify(this.triggers));
  return this.triggers && this.triggers.length > 0;
};

Trigger.prototype.getTrigger = function (config) {
  if (config && config.triggers) {
    Log.i(TAG, "config.triggers = " + config.triggers);
    try {
      var trigger = config.triggers;
      Log.i(TAG, "getTrigger trigger = " + JSON.stringify(trigger));
      return trigger;
    } catch (err) {
      Log.e(TAG, "getTrigger err = " + err);
    }
  }
  return [];
};

Trigger.prototype.getCrontab = function () {
  for (var trigger of this.triggers) {
    if (trigger.type === 'cron') {
      Log.d(TAG, "trigger[\"content\"].cron = " + trigger["content"].cron);
      return trigger["content"].cron;
    }
  }
  return null;
};

Trigger.prototype.hasCrontab = function () {
  return this.triggers.some(trigger => trigger.type === 'cron');
};

Trigger.prototype.startCrontab = function () {
  var self = this;
  var content = this.getCrontab();
  Log.i(TAG, "startCrontab begin..content =" + content);
  if (!content || content === "") {
    return false;
  }
  this.crontab = nodeSchedule.scheduleJob(content, () => {
    Log.i(RUN_TAG, `Timer trigger jobid ${this.jobId} begin to execute`);
    var params = {};
    params.event = {};
    params.event.type = "cron";
    params.event.payload = {};
    params.event.provider = {"groupId": this.groupId};
    this.setParams(params);
    this.execute();
  });
  if (!this.crontab) {
    Log.e(TAG, ` parse 'cron' setting failed.`);
  }
  return this.crontab;
};

Trigger.prototype.stopCrontab = function () {
  Log.i(TAG, "stopCrontab begin...");
  if (this.crontab) {
    nodeSchedule.cancelJob(this.crontab);
  }
  return true;
};

/**
 * [
 {
     "type":"device",
     "content":{
         "type" : "event",
         "productKey":"aa",
         "deviceName":"bb",
         "eventName":"cc",
         "key"       : "xx",
         "compareType": ">",
         "compareValue": "224"
     }
 },
 {
     "type":"device",
     "content":{
         "type": "propertyChange",
         "productKey":"aa",
         "deviceName":"bb",
         "propertyName":"xx",
         "compareType": "<",
         "compareValue": 234,
     }
 },
 {
     "type":"product",
     "content":{
         "type" : "event",
         "productKey":"aa",
         "eventName":"cc",
         "tags":[
             {"key": "xx", "value":""}
         ]
     }
 },
 {
     "type":"product",
     "content":{
         "type" : "propertyChange"
         "productKey":"aa",
         "propertyName":"cc",
         "tags":[
             {"key": "xx", "value":""}
         ]
     }
 },
 {
     "type":"cron",
     "content":{
         "cron":"5 0 * * *"
     }
 }
 {
    "type": "tag",
    "content": {
       "tags" : [{"key":"room"}, {"key":"deviceType", "value":"light"}]
     }
 }
 ]
 * @param type
 * @param event 1个event
 * @return {boolean}
 */
/*
info:     {"id":"obj1_abcde","eventName":"post","params":{"num":250},"eventType":0}
 */
Trigger.prototype.handle = function (info) {
  Log.i(TAG, this.jobId + " handle info = " + JSON.stringify(info));
  var groupConfig = this.groupConfig;
  if (!this.triggers || !info || !info.eventType || !info.id) {
    Log.w(TAG, "input empty, will return! this.triggers = " + JSON.stringify(this.triggers));
    return;
  }
  Log.i(TAG, "jobId " + this.jobId + " eventIds = " + this.eventIds + "\n propertyChangeIds = " + this.propertyChangeIds);
  var type = info.eventType;
  for (var param in info.params) {
    if ((info.params)[param] && typeof (info.params)[param] === 'object') {
      (info.params)[param] = (info.params)[param].value;
    }
  }
  if (type === Trigger.TRIGGER_TYPE.event && this.eventIds.indexOf(info.id) == -1) {
    return;
  } else if (type === Trigger.TRIGGER_TYPE.propertyChange && this.propertyChangeIds.indexOf(info.id) == -1) {
    return;
  }
  try {
    var list = info.id.split("_");
    var productKey = list[0];
    var deviceName = list[1];
    var tags;
    if (groupConfig && groupConfig.devices) {
      groupConfig.devices.forEach((device) => {
        if (device.productKey === productKey && device.deviceName === deviceName) {
          tags = device.tags;
        }
      });
    }
    for (var trigger of this.triggers) {
      var ids = this.parseTriggerId(trigger, groupConfig);
      if (ids.indexOf(info.id) === -1) {
        Log.i(TAG, `TriggerId don't hava ${info.id}, TriggerId = ${ids}`);
        continue;
      }
      if (type === Trigger.TRIGGER_TYPE.event && (trigger.content.type === 'event' || trigger.type === 'tag')) {
        var compareType = trigger.content.compareType;
        var key = trigger.content.key;
        if (trigger.type === 'tag' || (info.eventName === trigger.content.eventName && (!trigger.content.compareValue || (info.params && transformOperator((info.params)[key], compareType, trigger.content.compareValue))))) {
          this.send("deviceEvent", info, tags, ids, trigger.type);
          Log.i(TAG, "job deviceEvent execute end.");
          break;
        }
      } else if (type === Trigger.TRIGGER_TYPE.propertyChange && (trigger.content.type === 'propertyChange' || trigger.type === 'tag')) {
        var compareType = trigger.content.compareType;
        var key = trigger.content.propertyName;
        if (trigger.type === 'tag' || (info.params && (info.params)[key] !== undefined && (!trigger.content.compareValue || (info.params && transformOperator((info.params)[key], compareType, trigger.content.compareValue))))) {
          this.send("devicePropertyChange", info, tags, ids, trigger.type);
          Log.i(TAG, "job devicePropertyChange execute end.");
          break;
        }
      }
    }
    console.log(TAG, "handle end.");
  } catch (err) {
    Log.e(TAG, "handle err = " + err);
  }
};

Trigger.prototype.startFC = function (event, context) {
  Log.i(TAG, "startFC begin event = ", JSON.stringify(event));
  if (!event) {
    event = {};
  }
  event.provider = {
    "groupId": this.groupId,
  };
  this.job.event = event;
  this.job.context = context;
  Log.m(RUN_TAG, `[${this.job.sceneId}] Trigger begin to execute event : `, JSON.stringify(this.job.event));
  this.execute();
};

Trigger.prototype.setParams = function (params) {
  this.job.event = params.event;
  this.job.context = params.context;
  Log.m(RUN_TAG, `[${this.job.sceneId}] ${this.job.event.type} Trigger begin to execute event : `, JSON.stringify(this.job.event));
};

Trigger.prototype.send = function (eventType, payload, tags, triggerIds, triggerType) {
  Log.i(RUN_TAG, `${eventType} trigger jobid ${this.jobId} begin to execute`);
  Log.i(RUN_TAG, "send eventType =" + eventType + " payload " + JSON.stringify(payload));
  var list = payload.id.split("_");
  var productKey = list[0];
  var deviceName = list[1];
  var params = {};
  params.event = {};
  params.event.type = eventType;
  params.event.payload = payload;
  params.event.provider = {
    "productKey": productKey,
    "deviceName": deviceName,
    "groupId": this.groupId,
    "tags": tags
  };
  if (triggerType === 'product') {
    params.event.bindingDevices = [];
    for (var id of triggerIds) {
      var list = id.split("_");
      params.event.bindingDevices.push({'productKey': list[0], 'deviceName': list[1]})
    }
  }
  this.setParams(params);
  this.execute();
};

/**
 * 算出trigger中触发因子的设备id
 * @param trigger
 * @param groupConfig
 * @return {Array}
 */
Trigger.prototype.parseTriggerId = function (trigger, groupConfig) {
  var ids = [];
  switch (trigger.type) {
    case 'device': {
      ids.push(trigger.content.productKey + "_" + trigger.content.deviceName);
      break;
    }
    case 'product' : {
      var tags = trigger.content.tags;
      var productKey = trigger.content.productKey;
      Log.d(TAG, "productKey = " + productKey);
      Log.d(TAG, "tags = " + tags);
      var idList = this.getTagIdsFromGroupConfig(groupConfig, tags);
      console.log(TAG, "idList = " + idList);
      for (var id of idList) {
        if (id.split("_")[0] === productKey) {
          ids.push(id);
        }
      }
      break;
    }
    case 'tag' : {
      var tags = trigger.content.tags;
      ids = ids.concat(this.getTagIdsFromGroupConfig(groupConfig, tags));
      break;
    }
    case 'cron' :
      break;
  }
  Log.i(TAG, "parseTriggerId ids = " + ids);
  return ids;
};

Trigger.prototype.updateTriggerIds = function (groupConfig) {
  Log.i(TAG, "updateTriggerIds call");
  var eventIds = [];
  var changeIds = [];
  var commonIds = [];
  var self = this;
  this.triggers.forEach((trigger) => {
    var ids = self.parseTriggerId(trigger, groupConfig);
    switch (trigger.type) {
      case 'device': {
        if (trigger.content.type === 'event') {
          eventIds = eventIds.concat(ids);
        } else if (trigger.content.type === "propertyChange") {
          changeIds = changeIds.concat(ids);
        }
        break;
      }
      case 'product' : {
        if (trigger.content.type === 'event') {
          eventIds = eventIds.concat(ids);
        } else if (trigger.content.type === "propertyChange") {
          changeIds = changeIds.concat(ids);
        }
        break;
      }
      case 'tag' : {
        commonIds = commonIds.concat(ids);
        break;
      }
    }
  });
  this.eventIds = eventIds.concat(commonIds);
  this.propertyChangeIds = changeIds.concat(commonIds);
  Log.i(TAG, "jobId " + this.jobId + " eventIds = " + this.eventIds + "\n propertyChangeIds = " + this.propertyChangeIds);
};

/**
 * 找到groupConfig 中同时复合[{}, {}] tags 列表的设备
 * @param groupConfig
 * @param tags
 * @return {Array}
 */
Trigger.prototype.getTagIdsFromGroupConfig = function (groupConfig, tags) {
  var ids = [];
  if (!groupConfig || !groupConfig.devices) {
    return ids;
  }
  if (!tags || tags.length === 0) {
    /* groupConfig.devices:
    [{"deviceName":"EdgeDeviceNodeA","productKey":"a1HtrQxTVw0"}]
     */
    var devices = groupConfig.devices;
    for (var i in devices) {
      ids.push(devices[i].productKey + "_" + devices[i].deviceName);
    }
    return ids;
  }
  groupConfig.devices.forEach((device) => {
    if (isContains(device.tags, tags)) {
      ids.push(device.productKey + "_" + device.deviceName);
    }
  });
  return ids;
};

Trigger.prototype.updateGroupConfig = function (type, groupConfig) {
  if (!groupConfig || Object.keys(groupConfig).length == 0) return;
  var groupId = groupConfig.groupId;
  Log.i(TAG, "updateGroupConfig type = " + type + "script groupId = " + this.groupId + " updateGroupConfig groupId =" + groupId);
  //TODO type DELETE 具体化
  if (type === 10201) {
    this.groupConfig = groupConfig;
    this.updateTriggerIds(this.groupConfig);
  }
  // if (type === 10204 && groupId === this.groupId) {
  //     this.groupConfig = null;
  // }
};

Trigger.prototype.getConfigObject = function () {
  var config;
  try {
    var configPath = this.job.path + "/config.json";
    Log.i(TAG, "configPath = " + configPath);
    var configStr = fs.readFileSync(configPath, "utf-8");
    Log.i(TAG, "getConfigStr = " + configStr);
    config = JSON.parse(configStr);
  } catch (e) {
    Log.e(TAG, "getConfigObject Exception = " + e.message);
  }
  return config;
};

Trigger.prototype.getGroupId = function () {
  if (this.config) {
    return this.config.groupId;
  }
  return null;
};

/**
 * 运算符转化
 * @param srcValue
 * @param compareType
 * @param desValue
 * @return {Object}
 */
function transformOperator(srcValue, compareType, desValue) {
  if (!srcValue || !compareType || !desValue) {
    return false;
  }
  switch (compareType) {
    case "between" : {
      if (Array.isArray(desValue) && desValue.length > 0 && srcValue <= desValue[desValue.length - 1] && srcValue >= desValue[0]) {
        return true;
      } else {
        return false;
      }
    }
    case "like" : {
      if (typeof srcValue === 'string' && typeof desValue === 'string') {
        return srcValue.indexOf(desValue) != -1 ? true : false;
      }
      return false;
    }
    case "in" : {
      if (Array.isArray(desValue)) {
        return desValue.indexOf(srcValue) != -1 ? true : false;
      }
      return false;
    }
    case "=" : {
      return srcValue === desValue;
    }
    case "==" : {
      return srcValue === desValue;
    }
    default : {
      return eval(srcValue + compareType + desValue);
    }
  }
};

/**
 * 判断aTags 是否包含bTags
 * @param {object} aTags
 * @param {object} bTags
 * @return {boolean}
 * eg:[{}, {}, {}] [{}, {}]
 */
function isContains(aTags, bTags) {
  if (aTags == null && bTags == null) {
    return true;
  } else if (!aTags || !bTags) {
    return false;
  }
  if (Array.isArray(aTags) && Array.isArray(bTags)) {
    var cntBTag = 0;
    for (var bTag of bTags) {
      for (var aTag of aTags) {
        var cnt = 0;
        for (var key in bTag) {
          if (aTag.hasOwnProperty(key) && aTag[key] === bTag[key]) {
            cnt++;
          }
        }
        if (cnt === Object.keys(bTag).length) {
          cntBTag++;
          break;
        }
      }
    }
    if (cntBTag === Object.keys(bTags).length) {
      return true;
    }
  }
  return false;
}

module.exports = Trigger;
